/*****************************************************
 *  
 *  Copyright 2009 Adobe Systems Incorporated.  All Rights Reserved.
 *  
 *****************************************************
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.1 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *   
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *   
 *  
 *  The Initial Developer of the Original Code is Adobe Systems Incorporated.
 *  Portions created by Adobe Systems Incorporated are Copyright (C) 2009 Adobe Systems 
 *  Incorporated. All Rights Reserved.
 *  
 *****************************************************/

package org.osmf.advertisementplugin
{
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.external.ExternalInterface;
	import flash.utils.Dictionary;
	import flash.utils.Timer;
	import flash.utils.getTimer;
	
	import flashx.textLayout.operations.PasteOperation;
	
	import org.osmf.containers.MediaContainer;
	import org.osmf.elements.ProxyElement;
	import org.osmf.elements.VideoElement;
	import org.osmf.events.AudioEvent;
	import org.osmf.events.BufferEvent;
	import org.osmf.events.ContainerChangeEvent;
	import org.osmf.events.DisplayObjectEvent;
	import org.osmf.events.PlayEvent;
	import org.osmf.events.TimeEvent;
	import org.osmf.events.TimelineMetadataEvent;
	import org.osmf.layout.LayoutMetadata;
	import org.osmf.layout.ScaleMode;
	import org.osmf.logging.Logger;
	import org.osmf.media.*;
	import org.osmf.metadata.TimelineMarker;
	import org.osmf.metadata.TimelineMetadata;
	import org.osmf.net.httpstreaming.HTTPDownloadManager;
	import org.osmf.traits.PlayState;
	import org.osmf.utils.OSMFSettings;

	/**
	 * The AdvertisementPluginInfo class provides the reference implementation for ad insertions.
	 */
	
	public class AdvertisementPluginInfo extends PluginInfo
	{
		public function AdvertisementPluginInfo()
		{
			super();
			// Register the external interface callback functions which we'll use in our interactive demo.			
			if (ExternalInterface.available)
			{
				ExternalInterface.addCallback("displayNonLinearAd", displayNonLinearAd);
				ExternalInterface.addCallback("displayLinearAd", displayLinearAd);
			}		
			fileDown = HTTPDownloadManager.getInstance(); //#### ADDED #### This isn't a very good idea, look at the cleanliness on the ctr...
		}
		
		/**
		 * Initialize the plugin. 
		 */ 
		override public function initializePlugin(resource:MediaResourceBase):void
		{
			// Read the plugin configuration. Use string literals for simplicity/readability reasons.
			mediaPlayer = resource.getMetadataValue("MediaPlayer") as MediaPlayer;
			mediaContainer = resource.getMetadataValue("MediaContainer") as MediaContainer;
			mediaFactory = resource.getMetadataValue(PluginInfo.PLUGIN_MEDIAFACTORY_NAMESPACE) as MediaFactory;
						
			prerollURL = resource.getMetadataValue("preroll") as String;
			postrollURL = resource.getMetadataValue("postroll") as String;
			midrollURL = resource.getMetadataValue("midroll") as String;
			midrollTime = int(resource.getMetadataValue("midrollTime"));		
			oriMidroll=midrollTime;
			fileDown.receivedelta(20);
			overlayURL = resource.getMetadataValue("overlay") as String;
			overlayTime = int(resource.getMetadataValue("overlayTime"));
						
			// Expose so that we can disable the seek WORKAROUND for http://bugs.adobe.com/jira/browse/ST-397 
			// GPU Decoding issue on stagevideo: Win7, Flash Player version WIN 10,2,152,26 (debug)
			seekWorkaround = resource.getMetadataValue("seekWorkaround") != "false";
			
			if(prerollURL || postrollURL || midrollURL)
				fileDown.willplayAD=true;
//				
			
			if (prerollURL)
			{
				// NOTE: For progressive video the pause will not take effect immediately after playback has started.
				// So we need to pause the main media before it starts playing. To do this, we handle the 
				// BufferEvent.BUFFERING_CHANGE event, instead of PlayEvent.PLAY_STATE_CHANGE.
				// mediaPlayer.addEventListener(PlayEvent.PLAY_STATE_CHANGE, onPlayStateChange);
				
				mediaPlayer.addEventListener(BufferEvent.BUFFERING_CHANGE, onBufferChange);
			}
			
			if (postrollURL)
			{
				// TODO: Prebuffer the preroll before the playback completes.
				// The current implementation will likely change in future.
				mediaPlayer.addEventListener(TimeEvent.COMPLETE, onComplete);
			}			
			
			if (midrollURL && midrollTime > 0)
			{
				mediaPlayer.addEventListener(TimeEvent.CURRENT_TIME_CHANGE, onMidrollCurrentTimeChange);
			}
			
			if (overlayURL && overlayTime > 0)
			{
				mediaPlayer.addEventListener(TimeEvent.CURRENT_TIME_CHANGE, onOverlayCurrentTimeChange);
			}
			
			// Propagate the muted and volume changes from the video player to the advertisements.
			mediaPlayer.addEventListener(AudioEvent.MUTED_CHANGE, function(event:Event):void {
				for (var adPlayer:*  in adPlayers)
				{
					adPlayer.muted = mediaPlayer.muted;
				}
			});
			
			mediaPlayer.addEventListener(AudioEvent.VOLUME_CHANGE, function(event:Event):void {
				for (var adPlayer:*  in adPlayers)
				{
					adPlayer.volume = mediaPlayer.volume;
				}
			});
		}
		
		/*
		* Displays a linear advertisement. 
		* 
		* The method does not check if an ad is currently being played or not.
		* This is up to the caller to check. 
		* 
		* The ad will use the same layout as the main media.
		
		* @param url - the path to the ad media to be displayed.
		* @resumePlaybackAfterAd - indicates if the playback of the main media should resume after the playback of the ad.
		*/
		public function displayLinearAd(url:String, resumePlaybackAfterAd:Boolean = true):void
		{
//			fileDown.advert=true;
			displayAd(url, true, resumePlaybackAfterAd, true, null);
		}
		
		/**
		 * Displays a non-linear (overlay) advertisement. 
		 * If another ad is already being played, the new ad is added on top.
		 * 
		 * @param url - the path to the media
		 * @param layoutMetadata - information about the ad layout
		 */ 
		public function displayNonLinearAd(url:String, layoutInfo:Object):void
		{
			displayAd(url, false, false, true, layoutInfo);
			
		}	
		
		// Internals
		
		/**
		 * Utility function which plays an ad.
		 *  
		 * @param url - the path to the ad media to display.
		 * @param pauseMainMediaWhilePlayingAd - indicates if the main media needs to be paused while playing the ad.
		 * @param resumePlaybackAfterAd - indicates if the playback of the main media should resume after the playback of the ad.
		 * @param preBufferAd - indicates if we need to prebuffer the ad before playing it.
		 * @param layoutInfo - optional LayoutMetadata.
		 */ 
		private function displayAd(url:String, 
								   pauseMainMediaWhilePlayingAd:Boolean = true, 
//								   resumePlaybackAfterAd:Boolean = true, 
								   resumePlaybackAfterAd:Boolean = false,
								   preBufferAd:Boolean = true,
								   layoutInfo:Object = null):void
		{			
			// Set up the ad 
			var adMediaElement:MediaElement = mediaFactory.createMediaElement(new URLResource(url));	
			CONFIG::LOGGING
			{
					logger.debug("ad manager");
			}
			// Set the layout metadata, if present				
			if (layoutInfo != null)
			{
				var layoutMetadata:LayoutMetadata = new LayoutMetadata();
				for (var key:String in layoutInfo)
				{
					layoutMetadata[key] = layoutInfo[key];
				}		
				
				if (!layoutInfo.hasOwnProperty("index"))
				{
					// Make sure we add the last ad on top of any others
					layoutMetadata.index = adPlayerCount + 100;
				}
				
				adMediaElement.metadata.addValue(LayoutMetadata.LAYOUT_NAMESPACE, layoutMetadata);	
			}			
			
			var adMediaPlayer:MediaPlayer =  new MediaPlayer();		
			adMediaPlayer.media = adMediaElement;
			
			// Save the reference to the ad player, so that we can adjust the volume/mute of all the ads
			// whenever the volume or mute values change in the video player.
			adPlayers[adMediaPlayer] = true;
			adPlayerCount ++;
					
			adMediaPlayer.addEventListener(TimeEvent.COMPLETE, onAdComplete, true);			
			
			if (preBufferAd)
			{
				// Wait until the ad fills the buffer and is ready to be played.
				adMediaPlayer.muted = true;
				adMediaPlayer.addEventListener(BufferEvent.BUFFERING_CHANGE, onBufferingChange);
				function onBufferingChange(event:BufferEvent):void
				{
					if (event.buffering == false)
					{
						adMediaPlayer.removeEventListener(BufferEvent.BUFFERING_CHANGE, onBufferingChange);
						knownTime=fileDown.getTimetoSeek();
						logger.debug("known time "+knownTime);
						adMediaPlayer.seek(knownTime);
						playAd();
					}
				}		
			}
			else
			{
				playAd();
			}
			
			function playAd():void
			{		
				// Copy the player's current volume values
				adMediaPlayer.volume = mediaPlayer.volume;
				adMediaPlayer.muted = mediaPlayer.muted;
				
				if (pauseMainMediaWhilePlayingAd)
				{
					// Indicates to the player that we currently are playing an ad,
					// so the player can adjust its UI.
					mediaPlayer.media.metadata.addValue("Advertisement", url);
					
					// TODO: We assume that  playback pauses immediately,
					// but this is not the case for all types of content.
					// The linear ads should be inserted only after the player state becomes 'paused'.

//
//					adMediaPlayer.dispatchEvent(new TimeEvent(TimeEvent.COMPLETE,true,true));
					if(fileDown.prevMediaPlayer!=null){
						fileDown.prevMediaPlayer.pause();
					}
					mediaPlayer.pause();
					
					// If we are playing a linear ad, we need to remove it from the media container.
					if (mediaContainer.containsMediaElement(mediaPlayer.media))
					{
						mediaContainer.removeMediaElement(mediaPlayer.media);
					}
					else
					{
						// Wait until the media gets added to the container, so that we can remove it
						// immediately afterwards.
						mediaPlayer.media.addEventListener(ContainerChangeEvent.CONTAINER_CHANGE, onContainerChange);
//						mediaPlayer.seek(knownTime);
						function onContainerChange(event:ContainerChangeEvent):void
						{	
							if (mediaContainer.containsMediaElement(mediaPlayer.media))
							{
								mediaPlayer.media.removeEventListener(ContainerChangeEvent.CONTAINER_CHANGE, onContainerChange);
								mediaContainer.removeMediaElement(mediaPlayer.media);
							}
						}
					}					
				}
				
				// Add the ad to the container
				mediaContainer.addMediaElement(adMediaElement);
				fileDown.prevMediaPlayer =  adMediaPlayer;		
			}
			
			function onAdComplete(event:Event):void
			{
				var adMediaPlayer:MediaPlayer = event.target as MediaPlayer;
				adMediaPlayer.removeEventListener(TimeEvent.COMPLETE, onAdComplete);
				
				// Romove the ad from the media container
				mediaContainer.removeMediaElement(adMediaPlayer.media);
				
				// Remove the saved references
				adPlayerCount --;
				delete adPlayers[adMediaPlayer];					
				
				if (pauseMainMediaWhilePlayingAd)
				{					
					// Remove the metadata that indicates that we are playing a linear ad. 
					mediaPlayer.media.metadata.removeValue("Advertisement");
					
					// Add the main video back to the container.
					mediaContainer.addMediaElement(mediaPlayer.media);

				}
				
				if (pauseMainMediaWhilePlayingAd && resumePlaybackAfterAd)
				{	
					// WORKAROUND: http://bugs.adobe.com/jira/browse/ST-397 - GPU Decoding issue on stagevideo: Win7, Flash Player version WIN 10,2,152,26 (debug)
					if (seekWorkaround && mediaPlayer.canSeek)
					{
						mediaPlayer.seek(mediaPlayer.currentTime);
					}
					
					// Resume playback
					mediaPlayer.play();
					fileDown.willplayAD=false;
					fileDown.advert=false;
				}				
			}
		} 
		
		// Non-linear ad insertion
		
		/**
		 * Sample Non-Linear ad code. Uses the flash vars configuration
		 */ 
		private function onOverlayCurrentTimeChange(event:TimeEvent):void
		{			
			if (mediaPlayer.currentTime > overlayTime)
			{
				mediaPlayer.removeEventListener(TimeEvent.CURRENT_TIME_CHANGE, onOverlayCurrentTimeChange);
			
				// Hard-coded, for sample purposes. 
				var overlayMetadata:Object = {
					right: 10,
					bottom: 10,
					width: 200,
					height: 140,
					scaleMode: ScaleMode.STRETCH
				};
				
				displayNonLinearAd(overlayURL, overlayMetadata);
			}	
		}
		
		// Linear ad insertion
		
		/**
		 * Display the pre-roll advertisement.
		 */ 		
		private function onBufferChange(event:BufferEvent):void
		{
			if (event.buffering)
			{
				mediaPlayer.removeEventListener(BufferEvent.BUFFERING_CHANGE, onBufferChange);
				
				// Do not prebuffer the ad if playing a preroll ad.
				// Let the main content prebuffer while the ad is playing instead.
				displayAd(prerollURL, true, true, false, null);				
			}
		}
		
		/**
		 * Display the mid-roll ad.
		 */ 
		
		private function probabilisticChecker():Boolean
		{
			var nu:int=Math.floor(Math.random() * 11);

			if(nu<=5)
			{
				return true;
			}
			return false;
		}
		
		private function probabilisticStreamChoser_half():Boolean
		{
			var nu:int=Math.floor(Math.random() * 11);
			if(nu<=5)
			{
				return true;
			}
			return false;
		}
		
		private function probabilisticStreamChoser_equal():int
		{
			var nu:int=Math.floor(Math.random() * 7);
				return nu;
		}
		
		private function probabilisticStreamChoser_new_half():int
		{
			var nu:Number=Math.random();
			var w:Array=[0.285,0.285,0.142,0.142,0.068,0.068];
			if(nu<w[5]){
				return fileDown.positions[5];
			}else if(nu<(w[5]+w[4])){
				return fileDown.positions[4];
			}else if(nu<(w[5]+w[4]+w[3])){
				return fileDown.positions[3];
			}else if(nu<(w[5]+w[4]+w[3]+w[2])){
				return fileDown.positions[2];
			}else if(nu<(w[5]+w[4]+w[3]+w[2]+w[1])){
				return fileDown.positions[1];
			}else if(nu<(w[5]+w[4]+w[3]+w[2]+w[1]+w[0])){
				return fileDown.positions[0];
			}
			return fileDown.positions[0];
		}
		
		private function probabilisticStreamChoser_zipf():int
		{
			var nu:Number=Math.random();
			var w:Array=[0.409,0.204,0.135,0.102,0.081,0.065];
			if(nu<w[5]){
				return fileDown.positions[5];
			}else if(nu<(w[5]+w[4])){
				return fileDown.positions[4];
			}else if(nu<(w[5]+w[4]+w[3])){
				return fileDown.positions[3];
			}else if(nu<(w[5]+w[4]+w[3]+w[2])){
				return fileDown.positions[2];
			}else if(nu<(w[5]+w[4]+w[3]+w[2]+w[1])){
				return fileDown.positions[1];
			}else if(nu<(w[5]+w[4]+w[3]+w[2]+w[1]+w[0])){
				return fileDown.positions[0];
			}
			return fileDown.positions[0];
		}
		
		private function computeMidrollURL_mid():String
		{
			var template:String="http://x.x.x.x/vod/final_1.3";
			var options:Array=[0,1,2,3,4,5,6];
			var final_options:Array=new Array;
			var current:int=fileDown.currentStream;;
			var mode:String=fileDown.modicaPrefetchica;
			var chosenStream:String=new String;
			if(mode=="half"){
				var position:int=options.indexOf(current);
				if(position != 0 && position != 6){
					final_options.push(current-1);
					final_options.push(current+1);
				}else if(position == 0){
					final_options.push(6);
					final_options.push(1);
				}else if(position == 6){
					final_options.push(0);
					final_options.push(5);
				}
				var decision:Boolean=probabilisticStreamChoser_half();
				if(decision==true){
					if(final_options[0]==0){
						fileDown.vid_indexer=0;
						setStream=0;
						return template+".f4m";
					}else{
						chosenStream=final_options[0].toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}
				}else{
					if(final_options[1]==0){
						fileDown.vid_indexer=0;
						setStream=0;
						return template+".f4m";
					}else{
						chosenStream=final_options[1].toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}
				}
			}else if(mode=="equal"){
				logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]+" "+options[6]);
				logger.debug("current "+current);
				options.splice(options.indexOf(current),1);
				logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]);
				var decisione:int=probabilisticStreamChoser_equal();
				while(decisione==current){
					decisione=probabilisticStreamChoser_equal();
				}
				var val:int=decisione;
				if(decisione==0){
					fileDown.vid_indexer=0;
					setStream=0;
					return template+".f4m";
				}else if(decisione==1){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==2){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==3){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==4){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==5){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==6){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}
			}else if(mode=="new_half"){
				logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]+" "+options[6]);
				logger.debug("current "+current);
				options.splice(options.indexOf(current),1);
				logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]);
				var decisione:int=probabilisticStreamChoser_new_half();
				while(decisione==current){
					decisione=probabilisticStreamChoser_new_half();
				}
				var val:int=decisione;
				if(decisione==0){
					fileDown.vid_indexer=0;
					setStream=0;
					return template+".f4m";
				}else if(decisione==1){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==2){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==3){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==4){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==5){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==6){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}
			}else if(mode=="zipf"){
				logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]+" "+options[6]);
				logger.debug("current "+current);
				options.splice(options.indexOf(current),1);
				logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]);
				var decisione:int=probabilisticStreamChoser_zipf();
				while(decisione==current){
					decisione=probabilisticStreamChoser_zipf();//NEEDS TO BE ZIPF ORIGINALLY!!!!!
				}
				var val:int=decisione;
				if(decisione==0){
					fileDown.vid_indexer=0;
					setStream=0;
					return template+".f4m";
				}else if(decisione==1){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==2){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==3){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==4){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==5){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}else if(decisione==6){
					chosenStream=val.toString();
					fileDown.vid_indexer=Number(chosenStream);
					setStream=Number(chosenStream);
					return template+"_"+chosenStream+".f4m";
				}
			}
			return template;
		}
		
		private function computeMidrollURL_init():String
		{
			var template:String="http://x.x.x.x/vod/final_1.3";
			var options:Array=[0,1,2,3,4,5,6];
			var final_options:Array=new Array;
			var current:int=0;
			var mode:String=fileDown.modicaPrefetchica;
			var chosenStream:String=new String;
			if(mode=="half"){
				var position:int=options.indexOf(current);
				if(position != 0 && position != 6){
					final_options.push(current-1);
					final_options.push(current+1);
				}else if(position == 0){
					final_options.push(6);
					final_options.push(1);
				}else if(position == 6){
					final_options.push(0);
					final_options.push(5);
				}
				var decision:Boolean=probabilisticStreamChoser_half();
				if(decision==true){
					if(final_options[0]==0){
						fileDown.vid_indexer=0;
						setStream=0;
						return template+".f4m";
					}else{
						chosenStream=final_options[0].toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}
				}else{
					if(final_options[1]==0){
						fileDown.vid_indexer=0;
						setStream=0;
						return template+".f4m";
					}else{
						chosenStream=final_options[1].toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}
				}
			}else if(mode=="equal"){
				logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]+" "+options[6]);
				logger.debug("current "+current);
				options.splice(options.indexOf(current),1);
				logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]);
				var decisione:int=probabilisticStreamChoser_equal();
				while(decisione==current){
					decisione=probabilisticStreamChoser_equal();
				}
				var val:int=decisione;
				if(decisione==0){
						fileDown.vid_indexer=0;
						setStream=0;
						return template+".f4m";
					}else if(decisione==1){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==2){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==3){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==4){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==5){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==6){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}
				}else if(mode=="new_half"){
					logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]+" "+options[6]);
					logger.debug("current "+current);
					options.splice(options.indexOf(current),1);
					logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]);
					var decisione:int=probabilisticStreamChoser_new_half();
					while(decisione==current){
						decisione=probabilisticStreamChoser_new_half();
					}
					var val:int=decisione;
					if(decisione==0){
						fileDown.vid_indexer=0;
						setStream=0;
						return template+".f4m";
					}else if(decisione==1){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==2){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==3){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==4){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==5){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==6){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}
				}else if(mode=="zipf"){
					logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]+" "+options[6]);
					logger.debug("current "+current);
					options.splice(options.indexOf(current),1);
					logger.debug(options[0]+" "+options[1]+" "+options[2]+" "+options[3]+" "+options[4]+" "+options[5]);
					var decisione:int=probabilisticStreamChoser_zipf();
					while(decisione==current){
						decisione=probabilisticStreamChoser_zipf();
					}
					var val:int=decisione;
					if(decisione==0){
						fileDown.vid_indexer=0;
						setStream=0;
						return template+".f4m";
					}else if(decisione==1){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==2){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==3){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==4){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==5){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}else if(decisione==6){
						chosenStream=val.toString();
						fileDown.vid_indexer=Number(chosenStream);
						setStream=Number(chosenStream);
						return template+"_"+chosenStream+".f4m";
					}
				}
			return template;
		}
		
		private function recompute_positions():Array
		{
			var pointer:int=0;
			var compare:Array=[0,1,2,3,4,5,6];
			var final:Array=new Array;
			for(var i:int=0;i<compare.length;i++){
				if(fileDown.positions.indexOf(compare[i]) < 0){
					pointer=i;
				}
			}
			for(var i:int=pointer+1;i<compare.length;i++){
				final.push(compare[i]);
			}
			for(var i:int=0;i<pointer;i++){
				final.push(compare[i]);
			}
			return final;
		}
		
		private function midrollChecker():void
		{
			var returned:Boolean=probabilisticChecker();
			if ((mediaPlayer.currentTime > midrollTime) && returned==true)
			{	
				logger.debug("nu nu true "+" "+midrollTime);
				mediaPlayer.removeEventListener(TimeEvent.CURRENT_TIME_CHANGE, onMidrollCurrentTimeChange);
				midrollURL=computeMidrollURL_init();
				displayLinearAd(midrollURL);
				fileDown.currentStream=setStream;
				knownTime=fileDown.getTimetoSeek();
				fileDown.switchInitiated(knownTime);
				fileDown.positions.splice(fileDown.positions.indexOf(fileDown.currentStream),1);
				fileDown.positions.push(fileDown.pastStream);
				fileDown.pastStream=fileDown.currentStream;
				logger.debug("positions list "+fileDown.positions[0]+" "+fileDown.positions[1]+" "+fileDown.positions[2]+" "+fileDown.positions[3]+" "+fileDown.positions[4]+" "+fileDown.positions[5]);
				fileDown.positions=recompute_positions();
				logger.debug("loading advertisement "+midrollURL+" "+setStream);
				var myTimer1:Timer = new Timer(oriMidroll*1000, 0); 
				myTimer1.addEventListener(TimerEvent.TIMER, repeatMidroll);
				myTimer1.start();
			}else if(mediaPlayer.currentTime > midrollTime && returned==false){
				midrollTime+=oriMidroll;
			}
		}
		
		private function onMidrollCurrentTimeChange(event:TimeEvent):void
		{
			midrollChecker();
		}
		
		public function repeatMidroll(event:TimerEvent):void
		{
			var returned:Boolean=probabilisticChecker();
			midrollTime+=oriMidroll;
			mediaPlayer.removeEventListener(TimeEvent.CURRENT_TIME_CHANGE, onMidrollCurrentTimeChange);
			if (returned==true){
				logger.debug("nu nu true "+" "+midrollTime);
				newmidrollURL=computeMidrollURL_mid();
				displayLinearAd(newmidrollURL);
				fileDown.currentStream=setStream;
				knownTime=fileDown.getTimetoSeek();
				fileDown.switchInitiated(knownTime);
				fileDown.positions.splice(fileDown.positions.indexOf(fileDown.currentStream),1);
				fileDown.positions.push(fileDown.pastStream);
				fileDown.pastStream=fileDown.currentStream;
				fileDown.positions=recompute_positions();
				logger.debug("positions list "+fileDown.positions[0]+" "+fileDown.positions[1]+" "+fileDown.positions[2]+" "+fileDown.positions[3]+" "+fileDown.positions[4]+" "+fileDown.positions[5]);
				logger.debug("midroll url "+newmidrollURL);
				logger.debug("loading advertisement "+newmidrollURL+" "+setStream);
			}
		}
			
		/**
		 * Display the post-roll ad.
		 */ 
		private function onComplete(event:Event):void
		{
			mediaPlayer.removeEventListener(TimeEvent.COMPLETE, onComplete);
			
			// Resume the playback after the ad only if loop is set to true
			displayLinearAd(postrollURL, mediaPlayer.loop);
		}
		
		private var mediaPlayer:MediaPlayer;
		private var mediaContainer:MediaContainer;
		private var mediaFactory:MediaFactory;
		
		private var adPlayerCount:int = 0;
		private var setStream:int=new int;
		// Weak references for the currently playing ads
		private var adPlayers:Dictionary = new Dictionary(true);
		
		private var prerollURL:String;
		private var postrollURL:String;
		private var current_vid:String;
		private var midrollURL:String;
		private var midrollTime:int;
		private var oriMidroll:int
		private var knownTime:Number=0;
		
		private var overlayURL:String;
		private var overlayTime:int;
		private var seekWorkaround:Boolean = true;
		private static const logger:org.osmf.logging.Logger = org.osmf.logging.Log.getLogger("org.osmf.AdvertisementPluginInfo");
		private var fileDown:HTTPDownloadManager = null;
	}
}
